"""
Optimizations made on the input in list form, ie. before
conversion to a tree.
"""

from __future__ import generators
import re


collapse_ws_re = re.compile(r'[\x20\x0D\x0A\x09]+')

def collapse_whitespace (chunks):
    for chunk in chunks:
        if not chunk.tt:                # can't touch whitespace in <TT>
            chunk.set_text(collapse_ws_re.sub(' ', chunk.text))
        yield chunk


def merge_adjacent_whitespace (chunks):
    prev = None
    for chunk in chunks:
        #print '>', chunk
        if prev:
            if (not (prev.tt or chunk.tt) and
                prev.equivalent_whitespace(chunk)):
                # It's safe to move whitespace between these two chunks
                if prev.text[-1] == ' ' and chunk.text[0] == ' ':
                    chunk.set_text(chunk.text.lstrip())
                    
            if chunk.text:
                #print '<', prev
                yield prev
                prev = chunk

        else:                           # first pass through the loop
            prev = chunk

    if prev:
        #print '<', prev
        yield prev
    return


def merge_chunks (chunks):
    prev = None
    for chunk in chunks:
        if prev:
            if prev.equals_context(chunk): # can merge chunk with predecessor
                prev.append_text(chunk.text)
            else:
                yield prev
                prev = chunk

        else:                           # first pass through the loop
            prev = chunk

    if prev:
        yield prev

