from __future__ import generators
import sys

from sml_parser import VALID_TAGS


TAG_ENDS = {} # { tag_name : (open_tag, close_tag) }
for tag in VALID_TAGS:
    TAG_ENDS[tag] = ("<%s>" % tag, "</%s>" % tag)
TAG_ENDS[""] = ("", "") # pseudo root tag


class TagNode:

    def __init__(self, tag, children):
        self.tag = tag
        self.children = children

    def __repr__ (self):
        return "<%s at %08x: %s>" % (self.__class__.__name__,
                                     id(self), self.tag)

    def __str__(self):
        return "".join(self.render())
        
    def render (self, tag_ends=TAG_ENDS):
        """Generate a sequence of strings to be written out."""
        open_tag, close_tag = tag_ends[self.tag]
        yield open_tag
        for child in self.children:
            for s in child.render():
                yield s
        yield close_tag

    def dump (self, indent=''):
        print indent + ("tag: %s" % self.tag)
        indent += '  '
        for child in self.children:
            if isinstance(child, TagNode):
                child.dump(indent)
            else:
                print indent + ("txt: %r" % child.text)



def build_tree(parser):
    root = TagNode('', [])
    for chunk in parser:
        child = chunk
        if chunk.strong: child = TagNode('S',[child])
        if chunk.bold: child = TagNode('B',[child])
        if chunk.italics: child = TagNode('I',[child])
        if chunk.emph: child = TagNode('EM',[child])
        if chunk.tt: child = TagNode('TT',[child])
        if chunk.under: child = TagNode('U',[child])
        if chunk.size: child = TagNode(str(chunk.size),[child])
        if chunk.color: child = TagNode(str(chunk.color),[child])
        root.children.append(child)
    return root

